import datetime

import fabric.api as fab

@fab.task
def prepare(ref):
    """
    Create a git archive of the current working directory
    """
    fab.puts("Preparing release file for ref: %s" % ref)

    # this ensures our ref exists as fabric will bail out if it doesn't
    short_ref = fab.local("git rev-parse --short %s" % ref, capture=True)

    # generate a release filename
    now = datetime.datetime.utcnow()
    release_name = "%s-%s-%s" % (
        now.strftime("%Y.%m.%d"),
        now.strftime("%H.%M.%S"),
        short_ref
    )

    # use git to archive it locally
    fab.local("git archive --format=tar --prefix=%s/ %s | (cd /tmp; tar xf -)" % (
        release_name,
        short_ref,
    ))

    # now find any submodules
    fab.puts("Looking for submodules...")
    git_submodules = fab.local("find . -mindepth 2 -name .git -print | xargs grep -l '^gitdir:'", capture=True)
    for submodule in git_submodules.splitlines():
        submodule = submodule.lstrip("./").rstrip("/.git")

        submodule_ref = fab.local("git submodule status %s | awk '{print $1}'" % submodule, capture=True)

        # archive it
        fab.local("(cd %s; git archive --format=tar --prefix=%s/%s/ %s) | (cd /tmp; tar xf -)" % (
            submodule,
            release_name,
            submodule,
            submodule_ref
        ))

    # now pack up the main archive and clean up our working dir
    fab.local("(cd /tmp; tar zcf %s.tar %s && rm -fr %s)" % (
        release_name,
        release_name,
        release_name
    ))

    return release_name

@fab.task
def send(release_name):
    """
    Sends the specified release file to the servers

    :param release_name: The release name (returned from prepare_release)
    :return: None
    """
    releases_dir = fab.env.config.get('releases_dir', '~/releases/')
    fab.run("mkdir -p %s" % releases_dir)
    fab.put("/tmp/%s.tar" % release_name,
            "%s%s.tar" % (releases_dir, release_name))
    with fab.cd("~/releases"):
        fab.run("tar zxpf ./%s.tar" % release_name)
        fab.run("rm -f ./%s.tar" % release_name)

@fab.task
def make_current(release_name):
    """
    Makes the specified release the current one

    :param release_name: The release name (returned from prepare_release)
    :return:
    """
    releases_dir = fab.env.config.get('releases_dir', '~/releases/')
    current_symlink = fab.env.config.get('current_symlink', '~/current')
    fab.run("rm -f %s && ln -s %s/%s %s" % (current_symlink, releases_dir, release_name, current_symlink))

@fab.task
def clean_up():
    """
    Cleans up the release folder

    :return: None
    """
    keep_releases = fab.env.config.get('keep_releases', 10)
    releases_dir = fab.env.config.get('releases_dir', '~/releases/')

    # build the command line to cleanup the releases directory
    commands = [
        "find %s/* -maxdepth 0 -printf '%%T@ %%p\\n'" % releases_dir,
        "sort -k 1 -n",
        "awk '{print $2}'",
        "head -n -%d" % keep_releases,
        "xargs rm -fr"
    ]

    fab.run("|".join(commands))



import os

import fabric.api as fab

import yaml

@fab.task
def load(target, config_file):
    """
    Load into the environment the target config from the specified file

    :param target: The name of the target to load in the config
    :param config_file: The config file to load, relative to the fabfile
    :return: None
    """
    if not config_file.startswith("/"):
        fab_path = os.path.dirname(fab.env.real_fabfile)
        config_file = os.path.join(fab_path, config_file)

    if not os.path.isfile(config_file):
        return fab.abort("No config file found. Looked for: %s" % config_file)

    with open(config_file, 'r') as f:
        try:
            fab.env.config = yaml.load(f.read())
        except yaml.YAMLError as e:
            line = column = -1
            if hasattr(e, 'problem_mark'):
                line = e.problem_mark.line + 1
                column = e.problem_mark.column + 1
            return fab.abort("Error in %s YAML syntax. Line %d, column %d" % (
                config_file, line, column)
            )

    # make sure our deployment target exists
    if target not in fab.env.config:
        return fab.abort("The target '%s' is not defined in the config." % target)

    # reset our config to the target as the root
    fab.env.config = fab.env.config[target]

@fab.task
def show(target, config_file):
    """
    Loads and pretty prints the specified targets config (for debugging)
    """
    load(target, config_file)

    import pprint
    pprint.pprint(fab.env.config)


"""
This file provides a convenient way to use Fabric on your Vagrant box

Import the local vagrant instance (at the bottom) into your fabfile and
then you can call .run & .sudo and have them work on your current vagrant
instance
"""
import fabric.api as fab

import tempfile
import subprocess

class VagrantRunner(object):
    """
    A simple object that allows you to 'run' or 'sudo' on your Vagrant box
    """
    ssh_setup = False

    def setup_ssh(self):
        """
        Setup our local ssh config, only called once & when needed
        """
        if self.ssh_setup:
            return

        # create our ssh config
        self.ssh_config = tempfile.NamedTemporaryFile()

        # put our config in the file
        fab.local("vagrant ssh-config > %s" % self.ssh_config.name)

        self.orig_use_ssh_config = fab.env.use_ssh_config
        self.orig_ssh_config_path = fab.env.ssh_config_path

        self.ssh_setup = True

    def execute(self, command):
        """
        Shortcut used by run & sudo
        """
        self.setup_ssh()

        # make fabric use our config
        fab.env.use_ssh_config = True
        fab.env.ssh_config_path = self.ssh_config.name

        fab.execute(command, host='default')

        # restore the original values
        fab.env.use_ssh_config = self.orig_use_ssh_config
        fab.env.ssh_config_path = self.orig_ssh_config_path

    def run(self, *args, **kwargs):
        """
        Wrapper to fabric's run that runs on the current vagrant instance
        """
        @fab.task
        def vagrant_run():
            fab.run(*args, **kwargs)

        self.execute(vagrant_run)

    def sudo(self, *args, **kwargs):
        """
        Wrapper to fabric's sudo that runs on the current vagrant instance
        """
        @fab.task
        def vagrant_sudo():
            fab.sudo(*args, **kwargs)

        self.execute(vagrant_sudo)

# create an importable instance of our runner object
vagrant = VagrantRunner()



import os

import fabric.api as fab

import config
import release
import stages

@fab.task
def deploy(target, config_file="deploy.conf"):
    """
    Deploy your application to the specified 'target' in deploy.conf
    """
    # load our config file
    config.load(target, config_file)

    # setup our host roles
    if 'hosts' not in fab.env.config:
        return fab.abort("The target '%s' does not define any hosts." % target)

    roles = {
        'all': []
    }
    for hostname, hostconfig in fab.env.config['hosts'].iteritems():
        roles['all'].append(hostname)

        for role in hostconfig['roles']:
            if role not in roles:
                roles[role] = [hostname]
            else:
                roles[role].append(hostname)

    # set the roles with fabric
    fab.env.roledefs.update(roles)

    # prepare the release file
    if 'git_ref' not in fab.env.config:
        return fab.abort("The target '%s' does not specify a git_ref." % target)

    release_name = release.prepare(fab.env.config['git_ref'])

    # send the release to the hosts
    fab.execute(release.send, release_name, role='all')

    # clean up our local temp copy
    fab.local("rm -f /tmp/%s.tar" % release_name)

    # run our before tasks for this release
    stages.execute("before", release_name)

    # update the current symlink
    fab.execute(release.make_current, release_name, role='all')

    # run our after tasks for this release
    stages.execute("after", release_name)

    # clean up the releases directory
    fab.execute(release.clean_up, role='all')




#!/usr/bin/env python

from setuptools import setup

setup(name='garment',
      packages=['garment'],
      include_package_data=True,
      version='0.1',
      license="Apache License, Version 2.0",
      description='A collection of fabric tasks that roll up into a single deploy function. The whole process is coordinated through a single deployment configuration file named deploy.conf',
      long_description=open('README.rst').read(),
      author='Evan Borgstrom',
      author_email='evan@fatbox.ca',
      url='https://github.com/fatbox/garment',
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: Apache Software License',
          'Natural Language :: English',
          'Topic :: Software Development :: Libraries',
          'Topic :: Software Development :: Libraries :: Python Modules',
      ],
      install_requires=[
          'setuptools',
          'fabric>=1.8,<=1.9'
      ])
